#define DEBUG 1

/* 
 * main file for xfrac:
 *	Interactive mandelbrot drawing program.
 *
 *	Design goal:
 *	draw a mandlebot interactively
 *
 */

/*
 * Acknowlegements:
 * many thanks to the author of "tetris", for using proper X style, so that
 * I could learn.
 */

#include <stdio.h>
#include <stdlib.h>

#include <Intrinsic.h>
#include <StringDefs.h>
#include <Shell.h>
#include <Xaw/Form.h>
#include <Xaw/Simple.h>
#include <Xaw/Command.h>
#include <Xaw/Dialog.h>

  /* Intrinsic also does Xlib.h, Xutil.h, Xresource.h, Xos.h,
   *                     Xfuncproto.h
   */


#define MAIN 1
#include "globals.h"
#undef MAIN

#include "icon.xbm"

void quit(){
	XFreePixmap(display,mandelbrotPixmap);
	XtCloseDisplay(display);
}

static char * fallbacks[] = {
	"XMandel*mandel.width:  400",
	"XMandel*mandel.height: 400",
	"XMandel*mandel.background: black",
	"XMandel*form.width:    100",
	"XMandel*form.height:   400",
	"XMandel*form.background: white",
	NULL
};


XrmOptionDescRec optionList[] = {
	{"-display", ".display", XrmoptionSepArg, (caddr_t) NULL},
	{"-geometry", "*geometry", XrmoptionSepArg, (caddr_t) NULL},
	{"-iconGeometry", "*iconGeometry", XrmoptionSepArg, (caddr_t) NULL},
	{"-background", "*background", XrmoptionSepArg, (caddr_t) NULL},
	{"-bg", "*background", XrmoptionSepArg, (caddr_t) NULL},
	{"-foreground", "*foreground", XrmoptionSepArg, (caddr_t) NULL},
	{"-fg", "*foreground", XrmoptionSepArg, (caddr_t) NULL},
};


void QuitCallback(Widget widget,XtPointer data,XtPointer calldata)
/* ARGSUSED */
{
	XtCloseDisplay(display);
	exit(0);		
}

#include "patchlevel.h"

static char *version = VERSION;
void usage(){
	printf("%s\n",version);
	fflush(stdout);
	puts("");
	puts("    -fast:   uses quicker rendering algorythm");
	puts("");
	puts(" Xmandel draws a mandelbrot in a window, allowing certain parameters");
	puts("  to be changed. It does not draw the mandelbrot until the Draw button is pressed");
	puts(" It is possible to zoom in and out using the middle and right buttons");
	puts(" The left button will change the center of the picture");
	puts("");
	exit(0);
}

int main(int argc,char *argv[])
{
	Pixmap iconpixmap;
	srandom (time(NULL));
	mandelwidth = START_MANDEL_WIDTH;
	mandelheight = START_MANDEL_HEIGHT;
	toplevel = XtVaAppInitialize(&Context,"XMandel",
				   optionList,XtNumber(optionList),
				   &argc,argv,
				   fallbacks,
				   (ArgList) NULL,(Cardinal) NULL);

	CalculateColor = calculatePoint;
	if(argc>1){
		if(strcmp(argv[1],"-fast") == 0){
			CalculateColor = fastcalculatePoint;
			puts("Doing FAST calculation method");
		} else
			usage();
	}
	display = XtDisplay(toplevel);
	screen = XtScreen(toplevel);
	screennum = XScreenNumberOfScreen(screen);
	depth_of_screen = DefaultDepthOfScreen(screen);

	rootwindow = RootWindow(display,screennum);
	blackpixel = BlackPixel(display,screennum);
	whitepixel = WhitePixel(display,screennum);

	mandelbrotPixmap = XCreatePixmap(display,rootwindow,
		      mandelwidth,mandelheight,
		      depth_of_screen);

	MakeWidgets();
	XtRealizeWidget(toplevel);

	mainwindow = XtWindow(toplevel);
	iconpixmap = XCreateBitmapFromData(display,mainwindow,
			   icon_bits,icon_width,icon_height);

	XtVaSetValues(toplevel,XtNiconName,"xmandel",
				XtNiconPixmap,iconpixmap,
				NULL);

	defaultgc = DefaultGC(display,screennum);
	/* loadcolors also clears the pixmap */
	LoadColors();
	ReadValues(); /* initialize local variables from static strings */
	XtAppMainLoop(Context);
	return 0;
}
